/* Generated by compiling with any GCC version for MIPS Little Endian.
** GCC 5.4.0 is fine.

/usr/bin/mipsel-linux-gnu-gcc -g -O0 ./simple_mipsel.c -o ./simple_mipsel.elf
*/

void main(void)
{
    int a = 1, b = 2, res;

    res = a + b;

    return;
}
